;FUZZY INFERENCE EXAMPLES
; The rules call FLIE modules from within FuzzyCOPE.

(defrule DecompWithCrispIO
  ( crisp $?in )
=>
  ( bind $?out (dcmpfuzzy "a:\\loan.rul" $?in 0) )
  ( printout t "Decompositional inference = " $?out crlf )
)

(defrule FIDinference
  ( crisp $?in )
=>
  ( bind $?out (fidfuzzy "a:\\loan.rul" $?in) )
  ( printout t "FID inference = " $?out crlf )
)

(defrule LoadLoanInputSets
  ( loadfuzzy )
=>
  ( assert ( fuzzyInput
    0.0 0.0 0.5 1.0 1.0 0.2 0.0 0.0 0.0 0.0 0.0
    0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0
    1.0 0.7 0.3 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 )
  )
)

(defrule DecompFuzzyIO
  ( fuzzyInput $?in )
=>
  ( bind $?out (dcmpfuzzy "a:\\loan.rul" $?in 1) )
  ( printout t "Membership values = " $?out crlf )
)




; BACK PROPAGATION EXAMPLES
( defrule IrisBPNet
  ( bp )
=>
  ( bind $?status (newbp "a:\\bpfzz.wgt" 14 30 7) )
  ( printout t "Status (1 is ok) = " $?status crlf )
  ( assert (bpexists) )
)


( defrule IrisBPNet2
  ( bp )
=>
  ( bind $?status (newbp "a:\\bp.wgt" 4 10 3) )
  ( printout t "Status (1 is ok) = " $?status crlf )
  ( assert (bpexists2) )
)


( defrule TrainIrisBP
  ( bpexists )
  ( epochs ?e )
=>
  ( bind $?error (trainbp "a:\\bpfzz.wgt" "a:\\xorfz7.trn" ?e 0.5 0.01 0.001) )
  ( printout t "Training error = " $?error crlf )
)

( defrule TrainIrisBP2
  ( bpexists2 )
  ( epochs ?e )
=>
  ( bind $?error (trainbp "a:\\bp.wgt" "a:\\bpiris.trn" ?e 0.5 0.01 0.001) )
  ( printout t "Training error = " $?error crlf )
  ( assert (bptrained) )
)

( defrule RecallIrisBP1
  ( bptrained )
=>
  ( bind $?status (recallbp "a:\\bp.wgt" "a:\\bpiris.rcl" "a:\\bpiris.out") )
  ( printout t "Output status (1=ok) = " $?status crlf )
)


( defrule RecallIrisBP2
  ( bptrained )
=>
  ( bind $?out (recallbp "a:\\bp.wgt" "a:\\bpiris.rcl" ) )
  ( printout t "Output activations are " $?out crlf )
)

( defrule LoadBPMF_2Sets
  ( bptrained )
=>
  ( assert ( BPInputSets
	     0.0 0.9 0.6 0.3
	     1.0 0.2 0.2 0.8 )
  )
)

( defrule RecallIrisBP3
  ( BPInputSets $?in)
=>
  ( bind $?out (recallbp "a:\\bp.wgt" $?in) )
  ( printout t "Output activations are " $?out crlf )
)



; KOHONEN EXAMPLES
( defrule KohNet
  ( koh )
=>
  ( bind $?status (newkoh "a:\\koh.wgt" 4 6) )
  ( printout t "Status (1=ok) = " $?status crlf )
  ( assert (kohexists) )
)

( defrule TrainKoh
  ( kohexists )
  ( period ?p )
=>
  ( bind $?status (trainkoh "a:\\koh.wgt" "a:\\kohiris.trn" "0.5" 2 ?p) )
  ( printout t "Status (1=ok) = " $?status crlf )
  ( assert (kohtrained) )
)

( defrule LabelKohonen_FileIn_FileOut
  ( kohtrained )
=>
  ( bind $?status (labelkoh "a:\\koh.wgt" "a:\\kohiris.lbl" "a:\\kohiris.out") )
  ( printout t "Output status (1=ok) = " $?status crlf )
)

( defrule LabelKohonen_FileIn_MFOut
  ( kohtrained )
=>
  ( bind $?out (labelkoh "a:\\koh.wgt" "a:\\kohiris.lbl" ) )
  ( printout t "Output coordinates1 are " $?out crlf )
)

( defrule LoadKohonenLabelMF_2sets
  ( kohtrained )
=>
  ( assert ( kohInputSets
       0.662338 0.875 0.202899 0.08  "Virginica"
       0.818182 0.825 0.869565 1     "Sertosa"   )
  )
)

( defrule LabelKohonen_MFIn_MFOut
  ( kohInputSets $?in )
=>
  ( bind $?out (labelkoh "a:\\koh.wgt" $?in ) )
  ( printout t "Output coordinates are " $?out crlf )
)


; DATA ANALYSIS
( defrule normaliseIrisData
  ( norm )
=>
  ( bind $?status ( normalise "a:\\kohiris.trn" "a:\\nrmIris.trn" 1 ) )
  ( printout t "Status (1=ok) = " $?status crlf )
)

( defrule  fuzzifyIrisData
  ( fuzz )
=>
  ( bind $?status ( fuzzify "a:\\kohiris.trn" "a:\\fzIris.trn" 3 ) )
  ( printout t "Status (1=ok) = " $?status crlf)
)

( defrule  ExtractIrisRules
  ( getrules )
=>
  ( bind $?status ( extract "a:\\bpirisf3.wgt" "a:\\fzIris.rul" 1 0.5 0 ) )
  ( printout t "Status (1=ok) = " $?status crlf)
)
