; Rules to test all of the new functions added to fuzzy cope.
;=============================================================
;
; Tests for the backpropagation network (standard, fuzzy and rule 
; extraction) require two data files containing SE40 data.  
; These files are... 
;
;                cosetr21.raw     and     cosets21.raw
;
;
; Tests for the kohonen require the files...
;
;                kohtrain.trn     and     kohlabel.lbl
;
; These contain data that will produce two groups in 3 dimensional space
; and label them "Group 1" and "Group 2".
;
;
; This test has been written to run off the A drive.  If it is 
; necessary to run it from somewhere else use a search and replace 
; function to change the paths.
 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                   ;
; Rules to create the fuzzy and normalised training and test sets   ;
; from the raw data files.  (note that the raw files have already   ;
; been time lagged).                                                ;
;                                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Normalises the data sets
(defrule nor_data
  ( nor )
=>
; Two files used in training and testing the standard back-prop
  ( normalise "a:\\cosetr21.raw"
              "a:\\cosetr21.trn" 0 )
  ( normalise "a:\\cosets21.raw"
              "a:\\cosets21.trn" 0 )

; Two files to test log10 and logE normalisation
  ( normalise "a:\\cosetr21.raw" 
              "a:\\log10.trn" 1 )
  ( normalise "a:\\cosets21.raw" 
              "a:\\logE.trn" 2 ) 
)



; Fuzzify the data sets
(defrule fuzzify_data
  ( fuz )
=>
  ; Two files used to test and train the neural 
  ; networks for rule extraction
  ( fuzzify   "a:\\cosetr21.raw"
              "a:\\fzsetr21.trn" 7 ) 
  ( fuzzify   "a:\\cosets21.raw"
              "a:\\fzsets21.trn" 7 )  
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                   ;
;             Rules to create train and perform recall from         ;
;             a back prop using 2 time lags absolute values.        ;
;                                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

( defrule two_lag_absolute_values
  (create_bp)
=>
  ( bind $?m (newbp "a:\\bp2lagab.wgt" 2 10 1))
  ( printout t "status ( 1 is success) = " $?m crlf )
)



( defrule train_two_lag_absolute_values_net
  (train_bp ?x)
=>
  ( printout t "Epochs to train for = " ?x crlf)
  ( bind $?m (trainbp "a:\\bp2lagab.wgt" 
                      "a:\\cosetr21.trn" 
                      ?x 0.01 0.03 0.0001 ))
  ( printout t "Training error = " $?m crlf)
) 


( defrule Recall_absolute_2_lag
  ( recall_bp )
=>
  ; normalied values should be used here
  ( bind $?in 0.8 0.7 )
  ( bind $?m (recallbp "a:\\bp2lagab.wgt" $?in) )
  ( printout t "Output activations are " $?m crlf )
)


( defrule Recall_absolute_2_lag_keyboard
  ( recall_keyboard $?in )
=>
  ( bind $?m (recallbp "a:\\bp2lagab.wgt" $?in) )
  ( printout t "Output activations are " $?m crlf )
)


( defrule Recall_from_file
  ( recall_from_file )
=>
  ; normalied values should be used here
  ( bind $?m (recallbp "a:\\bp2lagab.wgt" "a:\\2lagab.rcl" ) )
  ( printout t "Output activations are " $?m crlf )
)


( defrule Recall_to_file
  ( output_to_file )
=>
  ; normalied values should be used here
  ( bind $?m (recallbp "a:\\bp2lagab.wgt" "a:\\2lagab.rcl" "a:\\temp.out" ) )
  ( printout t "Output activations are " $?m crlf )
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                               ;
;            Rules to create train and get recall from          ;
;        a backprop using 2 fuzzy time lags of 7 membership     ;
;           functions each.  Then extract rules from it and     ;
;           perform inference over those rules.                 ;
;                                                               ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

( defrule create_fuzzy
  (create_fuzzy)
=>
  ( bind $?m (newbp "a:\\fuzzy2l.wgt" 14 30 7))
  ( printout t "status ( 1 is success) = " $?m crlf )
)


( defrule train_fuzzy
  (train_fuzzy ?x)
=>
  ( printout t "Epochs to train for = " ?x crlf)
  ( bind $?m (trainbp "a:\\fuzzy2l.wgt" 
                      "a:\\fzsetr21.trn" 
                      ?x 0.5 0.01 0.0001 ))
  ( printout t "Training error = " $?m crlf)
) 


( defrule Recall_absolute_fuzzy_2_lag
  ( recall_fuzzy )
=>
  ( bind $?in 0 0.8 0.6 0 0 0 0   0 0.7 0.6 0 0 0 0 )
  ( bind $?m (recallbp "a:\\fuzzy2l.wgt" $?in) )
  ( printout t "Output activations are " $?m crlf )
)


; Rule to extract fuzzy rule from the net trained on fuzzy data.
( defrule extraxt_fuzzy_rule_from_2_lag
  ( extract_fuzzy )
=>
  ( extract "a:\\fuzzy2l.wgt" "a:\\coprule.rul" 
            0 0.05 0 )
  ( printout t "The rules have been extracted" crlf)
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                               ;
;        Perform fuzzy inference over the extracted rules.      ;
;        Uses the fuzzy engine created as part of FuzzyCOPE     ;
;        rather than the one supplied as part of fuzzy CLIPS.   ;
;                                                               ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            
( defrule Fuzzify_Infer_Defuzzify
  ( fuzzy_fid  $?in )
=>
  ( bind $?out ( fidfuzzy "a:\\coprule.rul" $?in ))
  ( printout t "Result of rules = " $?out )
)


( defrule Decompositional_Inference_singleton
  ( fuzzy_decomp_single  $?in )
=>
  ( bind $?out ( dcmpfuzzy "a:\\coprule.rul" $?in 0  ))
  ( printout t "Result of rules using singleton value = " $?out )
)
 

( defrule Decompositional_Inference_fuzzy
  ( fuzzy_decomp  $?in )
=>
  ( bind $?out ( dcmpfuzzy "a:\\coprule.rul" $?in 1  ))
  ( printout t "Result of rules using fuzzy value = " $?out )
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                  ;
;         Rules to create train, and label a Kohonen network       ;
;                                                                  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

( defrule create_the_kohonen_network
  ( create_kohonen )
=>
  ( bind $?m (newkoh "a:\\kohonen.wgt" 4 30 ))
  ( printout t "status ( 1 is success) = " $?m crlf )
)


( defrule train_the_kohonen_network
  ( train_kohonen ?x )
=>
  ( printout t "Number of epochs to train for = " ?x crlf)
  ( bind $?m (trainkoh "a:\\kohonen.wgt" "a:\\kohtrain.trn" "0.5" 2 ?x ))
  ( printout t "status ( 1 is success) = " $?m crlf ) 
)


( defrule label_the_kohonen_net
  ( label_from_file)
=>
  ( bind $?XYcoords (labelkoh "a:\\kohonen.wgt" "a:\\kohlabel.lbl" ))
  ( printout t "X Y = " $?XYcoords )
)


( defrule label_the_kohonen_net_XY
  ( label_XY $?in)
=>
  ( bind $?XYcoords (labelkoh "a:\\kohonen.wgt" $?in ))
  ( printout t "X Y = " $?XYcoords )
)


( defrule label_the_kohonen_net_to_file
  ( label_to_file)
=>
  ( bind $?XYcoords (labelkoh "a:\\kohonen.wgt" "a:\\kohlabel.lbl" "a:\\labeled.out" ))
  ( printout t "Status (1 is success) = " $?XYcoords )
)
