;********************************************************************************
;                                 Hybrid system
;********************************************************************************
; Up untill this point all of the rules have been for administrative details such
; as creating normalised data sets, creating, training and extracting rules from 
; networks.  This next section implements a hybrid system that makes use of some of 
; the trained networks an extracted fuzzy rules as well as extra common sense rules.
;
; The appliction is a Discission Support System to make recomendations on whether or
; not to buy or sello shares based on output from the neural nets, input from the
; user, and a number of business rules.
;*********************************************************************************


( defrule hybrid_system
  (hybrid)
=>
  ; get input from user on the political and economic
  ; climates.
  ( printout t "Is the economic climate good? " )
  (bind ?x ( read t))
  ( bind ?res (str-compare ?x "yes"))
  ( if ( = ?res 0 )  then                   
                      (assert ( economic climate good))                  
    else           
                      (assert ( economic climate bad))               
  )
  ( printout t " " crlf )
  ( printout t "Is the political climate good? " )
  (bind $?x ( read t))
  ( bind ?res (str-compare ?x "yes"))
  ( if ( = ?res 0 ) then 
                      (assert ( political climate good))  
    else
                      (assert ( political climate bad))
  )
  ( printout t " " crlf )
  
  ; Get the current and one past value of the time
  ; series to be passed to the network.
  ( printout t "Note: The values entered here must be normalised." crlf crlf )
  ( printout t "      As a linear normalisation was used to train " crlf )
  ( printout t "      the network the same process must be applied " crlf )
  ( printout t "      to the inputs.  Scale them between 0 and 1 "  crlf )
  ( printout t "      using 3968 as the maximum and 1142 as the " crlf )
  ( printout t "      minimum. " crlf crlf )  
  ( printout t "Enter the current value of the index: ")
  ( bind ?current_price ( read t))
  ( printout t "       Yesterdays value of the index: ")
  ( bind ?yesterdays_price (read t))
  ( printout t crlf )
  ( bind $?net_input ?current_price ?yesterdays_price)
  ( bind ?predicted_price (recallbp "a:\\bp2lagab.wgt" 
                                      $?net_input) )
  ( printout t "Output activations are " ?predicted_price crlf )
  
  ; assert something to control program execution.
  ( assert (step_2))
  ( assert (current_price ?current_price))
  ( assert (predicted_price ?predicted_price))
)



; Categorise the predicted values from the neural network
( defrule hybrid_system_step_2_r1
  ( step_2 )
  (current_price ?cp)
  (predicted_price ?pp)
=>
  (if (< ?cp ?pp) then
     ( assert ( predicted value is up))
     ( assert ( step_3 ))
     ( printout t "Processing step 2 " crlf)
  )
)

( defrule hybrid_system_step_2_r2
  ( step_2 )
  (current_price ?cp)
  (predicted_price ?pp)
=>
  (if (= ?cp ?pp) then
    ( assert ( predicted value is same))
    ( assert ( step_3 ))
    ( printout t "Processing step 2" crlf)
  )
)

( defrule hybrid_system_step_2_r3
  ( step_2 )
  (current_price ?cp)
  (predicted_price ?pp)
=>
  (if (> ?cp ?pp) then
    ( assert ( predicted value is down))
    ( assert ( step_3 ))
    ( printout t "Processing step 2" crlf)
  )
)


; apply business rules
( defrule hybrid_system_step_3_r1
  ( step_3 )
  ( political climate good )
  ( economic climate good  )
  ( predicted value is up )
  
=>
  ( assert ( buy shares ) )
  ( assert ( end_run ) )
  ( printout t "Processing step 3" crlf )
)  

( defrule hybrid_system_step_3_r2
  ( step_3 )
  ( political climate bad )
  ( economic climate good  )
  ( predicted value is up )  
=>
  ( assert ( buy shares ) )
  ( assert ( end_run ) )
  ( printout t "Processing step 3" crlf )
)  


( defrule hybrid_system_step_3_r3
  ( step_3 )
  ( political climate good )
  ( economic climate bad  )
  ( predicted value is same )  
=>
  ( assert ( hold shares ) )
  ( assert ( end_run ) )
  ( printout t "Processing step 3" crlf )
)  

( defrule hybrid_system_step_3_r4
  ( step_3 )
  ( political climate bad )
  ( economic climate bad  )
  ( predicted value is up )  
=>
  ( assert ( sell shares ) )
  ( assert ( end_run ) )
  ( printout t "Processing step 3" crlf )
)  

( defrule hybrid_system_step_3_r5
  ( step_3 )
  ( political climate good )
  ( economic climate good  )
  ( predicted value is down )  
=>
  ( assert ( hold shares ) )
  ( assert ( end_run ) )
  ( printout t "Processing step 3" crlf )
)  


; End Run Rule.  The final rule in the hybrid system 
; that fires.  This rule prints out a recommendation
; of whether or not to buy, sell, or hold shares based
; on the users perception of the political and economic
; climates and the neural nets prediction for the next
; value of the SE40.
( defrule end_of_run_1
  (end_run)
  ( buy shares )
=>
  (printout t crlf crlf "Decision Made by system." crlf)
  (printout t "Recommendation is to buy shares." crlf)
)


( defrule end_of_run_2
  (end_run)
  ( sell shares )
=>
  (printout t crlf crlf "Decision Made by system." crlf)
  (printout t "Recommendation is to sell shares." crlf)
)


( defrule end_of_run_3
  (end_run)
  ( hold shares )
=>
  (printout t crlf crlf "Decision Made by system." crlf)
  (printout t "Recommendation is to hold shares." crlf)
)


