;
;	IRRIGATE v1.0
;	Irrigation Control System
;	
;	Warren J. Fraser
;	INFO331, Intelligent Information Systems
;	
;	April 1996
;


;
;	TEMPLATES ESTABLISHING FUZZY VARIABLES AND MEMBERSHIP FUNCTIONS
;

( deftemplate 		  AirTemperature
	0 40		  Degrees
	(
	( Cold		  ( z 0 20 ))
	( Warm		  ( PI 15 20 ))
	( Hot		  ( s 20 40 ))
	))

( deftemplate 		  SoilMoisture
	0 100		  Percentage
	(
	( WiltingPoint	  ( z 0 50 ))
	( Stable	  ( PI 45 50 ))
	( FieldCapacity	  ( s 50 100 ))
	))

( deftemplate 		  Evaporation
	0 100		  Percentage
	(
	( Low		  ( z 0 50 ))
	( Medium	  ( PI 45 50 ))
	( High		  ( s 50 100 ))
	))

( deftemplate 		  Irrigation
	-50 150		  Millimetres
	(
	( Low	 	  ( z -50 145 ))
	( Medium 	  ( PI 45 50 ))
	( High	 	  ( s -45 150 ))
	))


;
;	DEFINING INITIAL FACTS
;

( deffacts InitialPhase
	( InitialFact )
	)


;
;	DEFINING INTRODUCTION AND INPUT
;

( defrule Introduction
	( InitialFact )
	=>
	( printout t " " crlf)
	( printout t "                         IRRIGATE v1.0" crlf)
	( printout t " " crlf)
	( printout t "                         Irrigation Control System"crlf )
	( printout t " " crlf )
	( printout t "      Developed by :     Warren J. Fraser" crlf )
	( printout t "      Developed for:     INFO331, Intelligent Information Systems" crlf )
	( printout t "      Developed during:  April 1996" crlf )
	( printout t " " crlf )
	( printout t "      Following input of air temperature, soil moisture and evaporation" crlf )
	( printout t "      variables, IRRIGATE v1.0 will determine the appropriate hourly" crlf )
	( printout t "      application rate of water required for sustainable and cost-effective" crlf )
	( printout t "      pasture production" crlf )
	( printout t " " crlf )
	)

( defrule Air
	( InitialFact )
	=>
	( printout t " " crlf )
	( printout t "      ENTER:             Air temperature (degrees)" crlf )
	( printout t "                         " )
	( bind ?air ( read t ))

	( if ( <= ?air 10 ) then ( assert ( AirTemperature Cold )))
	( if ( and ( > ?air 10 ) ( < ?air 30 )) then ( assert ( AirTemperature Warm )))
	( if ( >= ?air 30 ) then ( assert ( AirTemperature Hot )))
	)

( defrule Soil
	( InitialFact )
	=>
	( printout t " " crlf )
	( printout t "      ENTER:             Soil moisture (%)" crlf )
	( printout t "                         " )
	( bind ?soil ( read t ))
	( if ( <= ?soil 25 ) then ( assert ( SoilMoisture WiltingPoint)))
	( if ( and ( > ?soil 25 ) ( < ?soil 75 )) then ( assert ( SoilMoisture Stable )))
	( if ( >= ?soil 75 ) then ( assert ( SoilMoisture FieldCapacity )))
	)

( defrule Evapor
	( InitialFact )
	=>
	( printout t " " crlf )
        ( printout t "      ENTER:             Evaporation (%)" crlf )
	( printout t "                         " )
	( bind ?evapor ( read t ))
	( if ( <= ?evapor 25 ) then ( assert ( Evaporation Low )))
	( if ( and ( > ?evapor 25 ) ( < ?evapor 75 )) then ( assert ( Evaporation Medium )))
	( if ( >= ?evapor 75 ) then ( assert ( Evaporation High )))
	)


;
;	RULE FOR DEFUZZIFICATION
;

( defrule Defuzzify
	( declare ( salience -1 ))
	?output <- ( Irrigation ? )
	=>
	( bind ?result ( moment-defuzzify ?output ))

	( printout t "   " crlf)
        ( printout t "      RESULT:            Apply " ?result " millimetres of water" crlf )
	( printout t "   " crlf)
	( printout t "   " crlf)
	)

;
;	RULES FOR LOW IRRIGATION
;

( defrule Low1
	( AirTemperature Warm )
	( SoilMoisture FieldCapacity ) 
	( Evaporation Low )
	=>
	( assert ( Irrigation Low ))
	)

( defrule Low2
	( AirTemperature Cold )
	( SoilMoisture Stable ) 
	( Evaporation Low )
	=>
	( assert ( Irrigation Low ))
	)

( defrule Low3
	( AirTemperature Cold )
	( SoilMoisture FieldCapacity ) 
	( Evaporation Medium )
	=>
	( assert ( Irrigation Low ))
	)

( defrule Low4
	( AirTemperature Cold )
	( SoilMoisture FieldCapacity ) 
	( Evaporation Low )
	=>
	( assert ( Irrigation Low ))
	)


;
;	RULES FOR HIGH IRRIGATION
;

( defrule High1
	( AirTemperature Warm )
	( SoilMoisture WiltingPoint ) 
	( Evaporation High )
	=>
	( assert ( Irrigation High ))
	)

( defrule High2
	( AirTemperature Hot )
	( SoilMoisture Stable ) 
	( Evaporation High )
	=>
	( assert ( Irrigation High ))
	)

( defrule High3
	( AirTemperature Hot )
	( SoilMoisture WiltingPoint ) 
	( Evaporation Medium )
	=>
	( assert ( Irrigation High ))
	)

( defrule High4
	( AirTemperature Hot )
	( SoilMoisture WiltingPoint ) 
	( Evaporation High )
	=>
	( assert ( Irrigation High ))
	)


;
;	RULES FOR MEDIUM IRRIGATION
;

( defrule Medium1
	( AirTemperature Warm )
	( SoilMoisture FieldCapacity )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium2
	( AirTemperature Warm )
	( SoilMoisture FieldCapacity )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium3
	( AirTemperature Warm )
	( SoilMoisture Stable )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium4
	( AirTemperature Warm )
	( SoilMoisture Stable )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium5
	( AirTemperature Warm )
	( SoilMoisture Stable )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium6
	( AirTemperature Warm )
	( SoilMoisture WiltingPoint )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium7
	( AirTemperature Warm )
	( SoilMoisture WiltingPoint )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium8
	( AirTemperature Hot )
	( SoilMoisture Stable )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium9
	( AirTemperature Cold )
	( SoilMoisture Stable )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium10
	( AirTemperature Hot )
	( SoilMoisture Stable )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium11
	( AirTemperature Cold )
	( SoilMoisture Stable )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium12
	( AirTemperature Cold )
	( SoilMoisture WiltingPoint )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium13
	( AirTemperature Hot )
	( SoilMoisture FieldCapacity )
	( Evaporation Medium )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium14
	( AirTemperature Hot )
	( SoilMoisture FieldCapacity )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium15
	( AirTemperature Hot )
	( SoilMoisture FieldCapacity )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium16
	( AirTemperature Hot )
	( SoilMoisture WiltingPoint )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium17
	( AirTemperature Cold )
	( SoilMoisture WiltingPoint )
	( Evaporation Low )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium18
	( AirTemperature Cold )
	( SoilMoisture WiltingPoint )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

( defrule Medium19
	( AirTemperature Cold )
	( SoilMoisture FieldCapacity )
	( Evaporation High )
	=>
	( assert ( Irrigation Medium ))
	)

