;
;  Fuzzy Temperature Control
;
;  This program prompts for the current temperature of the room and the desired
;  temperature of the room - it then gives you an indication of what the temperature
;   in the room will be after you have implemented the changes.
;  If you do not need to change the settings (ie the heating controls or the windows)
;   then the system will tell you that you don't need to do anything.
;
; Written: Some time in 1996


;
; Fuzzy Variables and Membership Functions
;

(deftemplate room_temp ; Current temperature in the room
    -1  41 degrees
    (
    ; Membership Fn's
     ( freezing (Z -1 14))
     ( cold (PI 6 11))
     ( nice (PI 3 17))
     ( warm (PI 6.5 23.5))
     ( cooking (S 22 41))
    )
)


(deftemplate thermostat_setting  ; Desired temperature for the room
     -1 41 degrees
     (
     ; Membership Fn's
     ( freezing (Z -1 14))
     ( cold (PI 6 11))
     ( nice (PI 3 17))
     ( warm (PI 6.5 23.5))
     ( cooking (S 22 41))
     )
)



(deftemplate temperature_adjust ; Degree and type of adjustment to windows and
				    ; heater settings
				    
     -6 6 degrees
     (
     ; Membership Fn's
     ( High_Cooling (Z -6 -4))
     ( Medium_Cooling (PI 2 -4))
     ( Low_Cooling (PI 2  -2))
     ( Systems_Off (PI 2 0))
     ( Low_Heating (PI 2  2))
     ( Medium_Heating (PI 2 4))
     ( High_Heating (S 4 6))
     )
)



;Deffacts

(deffacts initial-phase
      (initial-fact))


; DefRules
; This rule gets the person to input the current room temperature
; and passes this information to the room_temp variable.

(defrule current_temp_input
      (initial-fact)
     =>
      (printout t "Please enter the current temperature in the room" crlf)
      (printout t " i.e: between 0 and 40 degrees" crlf)
      (bind ?var (read t))
      (assert (room_temp (PI 0.5 ?var)))
)


(defrule desired_temp_input
      (initial-fact)
     =>
      (printout t "" crlf)
      (printout t "Please enter the desired temperature for the room" crlf)
      (printout t " i.e: between 0 and 40 degrees" crlf)
      (bind ?var (read t))
      (assert (thermostat_setting (PI 0.5 ?var)))
 )



; Rules to change temperature

; Rule 1
(defrule cold_room_desire_freezing
        (thermostat_setting  freezing)
        (room_temp cold)
      => 
        (assert (temperature_adjust Low_Cooling))
)

; Rule 2
(defrule nice_room_desire_freezing
        (thermostat_setting  freezing)
        (room_temp nice)
      => 
        (assert (temperature_adjust Medium_Cooling))
)


; Rule 3 and 4
(defrule warm_or_cooking_room_desire_freezing
        (thermostat_setting  freezing)
         (room_temp warm OR cooking)
      => 
        (assert (temperature_adjust High_Cooling))
)


; Rule 5
(defrule freezing_room_desire_cold
        (thermostat_setting  cold)
        (room_temp freezing)
      => 
        (assert (temperature_adjust Low_Heating))
)


; Rule 6
(defrule nice_room_desire_cold
        (thermostat_setting  cold)
        (room_temp nice)
      => 
        (assert (temperature_adjust Low_Cooling))
)


; Rule 7
(defrule warm_room_desire_cold
        (thermostat_setting  cold)
        (room_temp warm)
      => 
        (assert (temperature_adjust Low_Cooling))
)


; Rule 8
(defrule cooking_room_desire_cold
        (thermostat_setting  cold)
        (room_temp cooking)
      => 
        (assert (temperature_adjust High_Cooling))
)


; Rule 9
(defrule freezing_room_desire_nice
        (thermostat_setting  nice)
        (room_temp freezing)
      => 
        (assert (temperature_adjust Medium_Heating))
)


; Rule 10
(defrule cold_room_desire_nice
        (thermostat_setting  nice)
        (room_temp cold)
      => 
        (assert (temperature_adjust Low_Heating))
)


; Rule 11
(defrule warm_room_desire_nice
        (thermostat_setting  nice)
        (room_temp warm)
      => 
         (assert (temperature_adjust Low_Cooling))
)


; Rule 12
(defrule cooking_room_desire_nice
        (thermostat_setting  nice)
        (room_temp cooking)
      => 
         (assert (temperature_adjust Medium_Cooling))
)


; Rule 13
(defrule freezing_room_desire_warm
        (thermostat_setting  warm)
        (room_temp freezing)
      => 
        (assert (temperature_adjust High_Heating))
)


; Rule 14
(defrule cold_room_desire_warm
        (thermostat_setting  warm)
        (room_temp cold)
      => 
        (assert (temperature_adjust Medium_Heating))
)


; Rule 15
(defrule nice_room_desire_warm
        (thermostat_setting  warm)
        (room_temp nice)
      => 
        (assert (temperature_adjust Low_Heating))
)


; Rule 16
(defrule cookng_room_desire_warm
        (thermostat_setting  warm)
        (room_temp cooking)
      =>
        (assert (temperature_adjust Low_Cooling))
)


; Rule 17 and 18
(defrule freezing_or_cold_room_desire_cooking
        (thermostat_setting  cooking)
        (room_temp freezing OR cold)
      =>
         (assert (temperature_adjust High_Heating))
)


; Rule 19
(defrule nice_room_desire_cooking
        (thermostat_setting  cooking)
        (room_temp nice)
      =>
         (assert (temperature_adjust Medium_Heating))
)


; Rule 20
(defrule warm_room_desire_cooking
        (thermostat_setting  cooking)
        (room_temp warm)
      =>
         (assert (temperature_adjust Low_Heating))
)


; Rule 21
(defrule freezing_room_desire_freezing
        (thermostat_setting  freezing)
        (room_temp freezing)
      =>
        (assert (temperature_adjust Systems_Off))
)


; Rule 22
(defrule cold_room_desire_cold
        (thermostat_setting  cold)
        (room_temp cold)
      =>
        (assert (temperature_adjust Systems_Off))
)


; Rule 23
(defrule nice_room_desire_nice
        (thermostat_setting  nice)
        (room_temp nice)
      =>
        (assert (temperature_adjust Systems_Off))
)


; Rule 24
(defrule warm_room_desire_warm
        (thermostat_setting  warm)
        (room_temp warm)
      =>
        (assert (temperature_adjust Systems_Off))
)


; Rule 25
(defrule cooking_room_desire_cooking
        (thermostat_setting  cooking)
        (room_temp cooking)
      =>
        (assert (temperature_adjust Systems_Off))
)



; Defuzzify Rule

(defrule defuzzify
   ( declare (salience -1))
     ?cs <- (temperature_adjust ?)
  =>
    ( bind ?defuz (moment-defuzzify ?cs))
    (printout t "Level of heating  should now be nearing  " ?defuz crlf)
    (printout t "Have a nice life...           " crlf)
    ( if ( and ( >= ?defuz -6 ) ( < ?defuz -5 ))
	then (printout t "Open windows fully" crlf)
                   (printout t "and turn Heating off" crlf))
    ( if ( and ( >= ?defuz -5 ) ( < ?defuz -3 ))
	then  (printout t "Open windows half-way" crlf)
                    (printout t "and turn Heating off" crlf))
    ( if ( and ( >= ?defuz -3 ) ( < ?defuz -1 ))
	then (printout t "Open windows slightly" crlf)
                   (printout t "and turn Heating off" crlf))
    ( if ( and (>= ?defuz -1) ( < ?defuz 1 ))
	then (printout t "Things are fine as they are." crlf)
                   (printout t "You do not need to adjust " crlf)
                   (printout t "the window or heating settings." crlf)
                   (printout t "Have a nice day..." crlf))
    ( if ( and (> ?defuz 1) ( < ?defuz 3 ))
	then  (printout t "Turn heating on to low" crlf)
                    (printout t "and close all the windows" crlf))
    ( if ( and ( >= ?defuz 3 ) ( < ?defuz 5 ))
	then  (printout t "Turn heating on to medium" crlf)
                    (printout t "and close all the windows" crlf))
    ( if ( and ( >= ?defuz 5 ) ( < ?defuz 6 ))
	then  (printout t "Turn heating on to high" crlf)
                    (printout t "and close all the windows" crlf))
)


